@@ -26,6 +26,8 @@ class ScoreKeeper(object):
                  output_value_map=False,
                  metric=STRICT_ACCURACY,
                  ignore_label=None,
+                 prediction_grouping=None,
+                 has_prediction_out_group=True,
                  row_keys=[],
                  band_names=[],
                  print_keys=[],
@@ -40,6 +42,15 @@ class ScoreKeeper(object):
         self.row_keys=row_keys or []
         self.band_names=band_names or []
         self.nb_bands=len(self.band_names)
+        if prediction_grouping:
+            self.prediction_grouping=prediction_grouping
+            self.has_prediction_out_group=has_prediction_out_group
+            if has_prediction_out_group:
+                self.out_group=[
+                    i for i in range(self.nb_classes) 
+                    if i not in prediction_grouping]
+        else:
+            self.prediction_grouping=False
         self.band_importance=band_importance
         self._set_metric(metric,ignore_label)
         if print_keys is True:
@@ -98,14 +109,15 @@ class ScoreKeeper(object):
                 in self.output_reducer_map],axis=-1)
         if not isinstance(preds,list):
             preds=[preds]
-        preds=[tf.argmax(p,axis=-1) for p in preds]
+        preds=[self._prediction_cat(p) for p in preds]
         if self.output_value_map:
             preds=[proc.map_values(p,self.output_value_map) for p in preds]
-        for (ts,ps) in zip(targs,preds):
+        for head,(ts,ps) in enumerate(zip(targs,preds)):
             for i,(targ,pred,row) in enumerate(zip(ts,ps,rows)):
                 score_data={}
                 score_data['batch']=batch
                 score_data['image_index']=i
+                score_data['head_index']=head
                 if row is not False:
                     for k in self.row_keys:
                         score_data[k]=row[k]
@@ -149,6 +161,33 @@ class ScoreKeeper(object):
     #
     # INTERNAL
     #
+    def _prediction_cat(self,pred):
+        if self.prediction_grouping:
+            return self._grouped_argmax(pred)
+        else:
+            return tf.argmax(pred,axis=-1)
+
+
+    def _group_sum(self,arr,group):
+        garr=tf.gather(arr,group,axis=-1)
+        gsum=tf.math.reduce_sum(garr,axis=-1)
+        return garr, gsum
+
+
+    def _grouped_argmax(self,arr):
+        garr, gsum=self._group_sum(arr,self.prediction_grouping)
+        amx=tf.argmax(arr,axis=-1)
+        gamx=tf.argmax(garr,axis=-1)
+        gamx=tf.cast(tf.gather(self.prediction_grouping,gamx),'int64')
+        if self.has_prediction_out_group:
+            _, ogsum=self._group_sum(arr,self.out_group)
+            return tf.where(gsum>ogsum, x=gamx,y=amx)
+        else:
+            mx=tf.math.reduce_max(arr,axis=-1)
+            return tf.where(gsum>mx, x=gamx,y=amx)
+
+
+
     def _get_sum_stack(self,tnsr,vals):
         if isinstance(vals,int):
             out=tnsr[:,:,:,vals]
